==> kv5500-0.3.tar.gz: kv5500-0.3/src/kv5500_sms.cpp <==
/*****************************************************************************
 *
 *  kv5500_sms - read a sms from kv5500 like phone.
 *
 * Copyright (c) 2006-2007 Hyo geol, Lee
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 *   3. The name of the author may not be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 *
 ****************************************************************************/

#include <cstdio>
#include <cstring>

#include "kv5500.hpp"
#include "kv5500_sms.hpp"

using namespace KV5500;

/// buf[offset]             start of phone number, offset 12, size 32.
/// buf[offset + 32]        start of date, offset 12, size 5.
/// buf[offset + 37]        what is this 1 bytes mean? i assume it second of 
///                         date. my phone don't display second on sms.
/// buf[offset + 38]        start of message, offset 12, (38 = 32 + 5 + 1)
///
/// I cannot get total message number, so need duplicat test. I'll use
/// last 4 bytes of date section.(offset + 33~37).
int SMSScanner::Recv(UsbDev& udev)
{
    char buf[1000];
    const int ret = udev.BulkRead(buf, sizeof(buf), 5000);

    /// assume ret must be over 54 bytes.
    if(ret <= 54)
        return -1;

    if(CheckExist(buf))
        return -1;

    const char* const dateStr = GetSMSDateStr(buf);

    smsContainer_.push_back(SMS(GetString(buf, 32, 12, 32, 0x00),
                                dateStr,
                                GetString(buf, ret - 53, 50, ret - 53, 0x00)));
    delete [] dateStr;

    return ret;
}

bool SMSScanner::DoScan(struct usb_device* dev)
{
    assert(dev);

    bool rtn = false;

    try {

        UsbDev udev(dev);

        if(DescCheck(dev)) {

            if(!WakeUp(udev))
                throw std::exception();

            if(!Request(udev))
                throw std::exception();

            if(!Shutdown(udev))
                throw std::exception();

            rtn = true;
        }

    } catch(std::exception& e) {

        printf("%s\n", e.what());
    }

    for (int i = 0; i < dev->num_children; i++)
        rtn = rtn || Scan(dev->children[i]);

    return rtn;
}

bool SMSScanner::Request(UsbDev& udev)
{
    char buf[3] = { 0xfe, 0x14, 0x7e};
    /// I don't know total sms number yet. so big loop.
    for(unsigned int i = 0; i < 200; ++i) {
                
        if(udev.BulkWrite(buf, sizeof(buf), 5000) < 0) {

            printf("Got error while send.\n");

            return false;

        } else {

            if(Recv(udev) < 0)
                break;
        }
    }

    return true;
}

bool SMSScanner::CheckExist(const char* const buf)
{
    /// check set to test duplication. test 46~49 data. cf) date is 44~48, 
    /// 49 maybe second.
    int thisMsgId = 0;
    memcpy(&thisMsgId, buf + 46, sizeof(int));
    if(dateSet_.count(thisMsgId))
        return true;

    dateSet_.insert(thisMsgId);

    return false;
}

const char* KV5500::GetSMSDateStr(const char* const buf)
{
    /// get date, offset 44, size 5?
    ///
    /// max str length = 1 bytes integer * 5 + field delimiter * 4 + NUL
    ///                = 3 * 5 + 1 * 4 + 1 = 20
    const size_t dateSize = 20;
    char* date = new char[dateSize];

    if(snprintf(date, dateSize, "%02d/%02d/%02d %02d:%02d",
                static_cast<int>(buf[44]),
                static_cast<int>(buf[45]),
                static_cast<int>(buf[46]),
                static_cast<int>(buf[47]),
                static_cast<int>(buf[48])) <= 0) {

        date[0] = '\0';
    }

    return date;
}

bool KV5500::GetSMS(std::vector<SMS>& sms)
{
    Usb thisUsb;
    SMSScanner smsScanner;

    if(!thisUsb.Scan(smsScanner))
        return false;

    smsScanner.GetSMS(sms);

    return true;
}

==> kv5500-0.3.1.tar.gz: kv5500-0.3.1/src/kv5500_sms.cpp <==
/*****************************************************************************
 *
 *  kv5500_sms - read a sms from kv5500 like phone.
 *
 * Copyright (c) 2006-2007 Hyo geol, Lee
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 *   3. The name of the author may not be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 *
 ****************************************************************************/

#include <cstdio>
#include <cstring>

#include "kv5500.hpp"
#include "kv5500_sms.hpp"

using namespace KV5500;
using namespace HG;

/// buf[offset]             start of phone number, offset 12, size 32.
/// buf[offset + 32]        start of date, offset 12, size 5.
/// buf[offset + 37]        what is this 1 bytes mean? i assume it second of 
///                         date. my phone don't display second on sms.
/// buf[offset + 38]        start of message, offset 12, (38 = 32 + 5 + 1)
///
/// I cannot get total message number, so need duplicat test. I'll use
/// last 4 bytes of date section.(offset + 33~37).
int SMSScanner::Recv(UsbDev& udev)
{
    char buf[1000];
    const int ret = udev.BulkRead(buf, sizeof(buf), 5000);

    /// assume ret must be over 54 bytes.
    if(ret <= 54)
        return -1;

    if(CheckExist(buf))
        return -1;

    const ConstCharArrayPtr dateStr = GetSMSDateStr(buf);

    smsContainer_.push_back(SMS(GetString(buf, 32, 12, 32, 0x00),
                                dateStr.Get(),
                                GetString(buf, ret - 53, 50, ret - 53, 0x00)));
    return ret;
}

bool SMSScanner::Scan(struct usb_device* dev)
{
    assert(dev);

    bool rtn = false;

    try {

        UsbDev udev(dev);

        if(DescCheck(dev)) {

            if(!WakeUp(udev))
                throw std::exception();

            if(!Request(udev))
                throw std::exception();

            if(!Shutdown(udev))
                throw std::exception();

            rtn = true;
        }

    } catch(std::exception& e) {

        printf("%s\n", e.what());
    }

    for (int i = 0; i < dev->num_children; i++)
        rtn = rtn || Scan(dev->children[i]);

    return rtn;
}

bool SMSScanner::Request(UsbDev& udev)
{
    char buf[3] = { 0xfe, 0x14, 0x7e};
    /// I don't know total sms number yet. so big loop.
    for(unsigned int i = 0; i < 200; ++i) {
                
        if(udev.BulkWrite(buf, sizeof(buf), 5000) < 0) {

            printf("Got error while send.\n");

            return false;

        } else {

            if(Recv(udev) < 0)
                break;
        }
    }

    return true;
}

bool SMSScanner::CheckExist(const char* const buf)
{
    /// check set to test duplication. test 46~49 data. cf) date is 44~48, 
    /// 49 maybe second.
    int thisMsgId = 0;
    memcpy(&thisMsgId, buf + 46, sizeof(int));
    if(dateSet_.count(thisMsgId))
        return true;

    dateSet_.insert(thisMsgId);

    return false;
}

ConstCharArrayPtr KV5500::GetSMSDateStr(const char* const buf)
{
    /// get date, offset 44, size 5?
    ///
    /// max str length = 1 bytes integer * 5 + field delimiter * 4 + NUL
    ///                = 3 * 5 + 1 * 4 + 1 = 20
    const size_t dateSize = 20;
    char* date = new char[dateSize];

    if(snprintf(date, dateSize, "%02d/%02d/%02d %02d:%02d",
                static_cast<int>(buf[44]),
                static_cast<int>(buf[45]),
                static_cast<int>(buf[46]),
                static_cast<int>(buf[47]),
                static_cast<int>(buf[48])) <= 0) {

        date[0] = '\0';
    }

    return ConstCharArrayPtr(date);
}

bool KV5500::GetSMS(std::vector<SMS>& sms)
{
    Usb thisUsb;
    SMSScanner smsScanner;

    if(!thisUsb.Scan(smsScanner))
        return false;

    smsScanner.GetSMS(sms);

    return true;
}
